#
# Copyright (C) 2020 Arm Mbed. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
#
"""Exceptions raised by mbed tools."""


class ToolsError(Exception):
    """Base class for tools errors."""
